// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;

import server.dao.AclDAO;
import server.dao.DAOFactory;
import server.global.Constants;
import utils.HibernateSession;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Hashtable;

/**
 * The customtables are used in CmnServer to get jdbc-connection URLs for
 * tables and databases with individual customer data.
 *  
 * This Hibernate class exists so that
 * a) the customtables-table is documented.
 * b) Hibernate can create an empty table when used with hibernate.hbm2ddl.auto=update or create.
 *  
 * @author Ingo Wiarda
 *
 */

@Entity
@Table(name = "customtables")
public class CustomTable implements Serializable {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	static final DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
	@Id @GeneratedValue
	@Column(name = "id")
	private long 	id;
	
	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;
	
	@Column(name = "connstring",
			length = 512,
			nullable = false)
	private String connstring;

	@ManyToOne
	@JoinColumn(name = "acl_id",
				nullable = false
	)	
	private Acl acl;
	
	@Column(name = "jdbcdriver",
			length = 128,
			nullable = false)
	private String jdbcDriver;
	
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;
	
	public CustomTable(){
		
	}
	
	public CustomTable(String name, String connstring, String jdbcDriver, Acl acl){
		this.name = name;
		this.connstring = connstring;
		this.acl = acl;
		this.jdbcDriver = jdbcDriver;
	}
	
	public CustomTable(Hashtable<String,String> cmd){
			name		= cmd.get("name");
			connstring 	= cmd.get("connstring");			
			AclDAO aclDAO = daoFactory.getAclDAO(HibernateSession.getLocalEntityManager());
			acl 		= aclDAO.get(new Long(cmd.get("permissionid")));
	}
		
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return the connstring
	 */
	public String getConnstring() {
		return connstring;
	}

	/**
	 * @param connstring the connstring to set
	 */
	public void setConnstring(String connstring) {
		this.connstring = connstring;
	}

	/**
	 * @return the acl
	 */
	public Acl getAcl() {
		return acl;
	}

	/**
	 * @param acl the acl to set
	 */
	public void setAcl(Acl acl) {
		this.acl = acl;
	}

	public long getId() {
		return id;
	}
	
	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	/**
	 * @return the jdbcDriver
	 */
	public String getJdbcDriver() {
		return jdbcDriver;
	}

	/**
	 * @param jdbcDriver the jdbcDriver to set
	 */
	public void setJdbcDriver(String jdbcDriver) {
		this.jdbcDriver = jdbcDriver;
	}
	
}
