// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;

import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.DAOFactory;
import server.dao.ObjectSystemDataDAO;
import server.dao.RelationTypeDAO;
import server.data.ObjectSystemData;
import server.exceptions.CinnamonException;
import utils.HibernateSession;
import utils.ParamParser;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Hashtable;

@Entity
@Table(name = "relations",
		uniqueConstraints = {@UniqueConstraint(columnNames={"leftid", "rightid", "type_id"})}
)
public class Relation 
	implements Serializable{

	private static final long	serialVersionUID	= 1L;
	static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
	@Id @GeneratedValue
	@Column(name = "id")
	private long id;

	@ManyToOne(cascade = {})
	@JoinColumn(name = "type_id")	
	private RelationType type;
	
	@ManyToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
	@JoinColumn(name = "leftid",
				nullable = false)	
	private ObjectSystemData leftOSD;
	
	@ManyToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
	@JoinColumn(name = "rightid",
				nullable = false)	
	private ObjectSystemData rightOSD;

	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version;
	
	
	public Relation(){
		
	}
		
	public Relation(RelationType type, ObjectSystemData left, ObjectSystemData right){
		this.type = type;
		this.leftOSD = left;
		this.rightOSD = right;
	}
	
	// TODO: change api to receive relationtype_id instead of name.
	public Relation(Hashtable<String,String> cmd){
		EntityManager em = HibernateSession.getLocalEntityManager();

		String name = cmd.get("name");	
		RelationTypeDAO relationTypeDAO = daoFactory.getRelationTypeDAO(em);
		type = relationTypeDAO.findByName(name);
		Logger log=LoggerFactory.getLogger(this.getClass());if(type == null){
			log.debug("Could not find RelationType");
			throw new RuntimeException("error.relation_type.not_found");
		}
		
		Long leftId=  ParamParser.parseLong(cmd.get("leftid"), "error.param.left_id");
		Long rightId= ParamParser.parseLong(cmd.get("rightid"), "error.param.right_id");
		
		ObjectSystemDataDAO osdDAO = daoFactory.getObjectSystemDataDAO(em);
		rightOSD = osdDAO.get(Long.parseLong( cmd.get("rightid")));		
		if(rightOSD == null){
			log.debug("Could not find rightOSD with id " + rightId);
			// TODO: parametrize exception message.
//			throw new RuntimeException( "Object with ID " + rightId + " was not found in DB");
			throw new CinnamonException("error.right_id_object.not_found");
		}
		leftOSD = osdDAO.get(Long.parseLong( cmd.get("leftid")));
		if(leftOSD == null){
			log.debug("leftOSD with id " + leftId +"not found");
			// TODO: parametrize exception message.
			throw new CinnamonException("error.left_id_object.not_found");
		}
	}
	
	public long getId() {
		return id;
	}

	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	public ObjectSystemData getLeft() {
		return leftOSD;
	}

	public void setLeft(ObjectSystemData left) {
		this.leftOSD = left;
	}

	public ObjectSystemData getRight() {
		return rightOSD;
	}

	public void setRight(ObjectSystemData right) {
		this.rightOSD = right;
	}

	public RelationType getType() {
		return type;
	}

	public void setType(RelationType type) {
		this.type = type;
	}

	public void toXmlElement(Element root){
		Element relation = root.addElement("relation");
		relation.addElement("id").addText(String.valueOf(getId()) );
		relation.addElement("leftId").addText( String.valueOf(leftOSD.getId()));
		relation.addElement("rightId").addText( String.valueOf(rightOSD.getId()));
		relation.addElement("type").addText(type.getName());
	}
}
