// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;

import org.dom4j.Element;
import server.global.Constants;
import server.i18n.LocalMessage;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Hashtable;

@Entity
@Table(name = "relationtypes",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}
)
public class RelationType
	implements Serializable{
	
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;

	@Id @GeneratedValue
	@Column(name = "id")
	private long id;
	
	@Column (name="name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;
	
	@Column (name="description",
			length = Constants.DESCRIPTION_SIZE,
			nullable = false)
	private String description;
	
	/**
	 * If leftobjectprotected is true:
	 * the left object of a relation of this type may not be deleted as long
	 * as the relation exists.
	 */
	@Column  (name="leftobjectprotected",
			nullable = false)	
	private Boolean leftobjectprotected;
	
	/**
	 * If rightobjectprotect is true:
	 * the right object of a relation of this type may not be deleted as long
	 * as the relation exists.
	 */
	@Column  (name="rightobjectprotected",
			nullable = false)
	private Boolean rightobjectprotected;
		
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;
	
	public RelationType(){
		
	}
	
	public RelationType(String name, String description,
			Boolean leftobjectprotected, Boolean rightobjectprotected){
		this.name = name;
		this.description = description;
		this.leftobjectprotected = leftobjectprotected;
		this.rightobjectprotected = rightobjectprotected;
	}
	
	public RelationType(Hashtable<String,String> cmd){
		name = cmd.get("name");	
		description = cmd.get("description");
		leftobjectprotected = cmd.get("leftobjectprotected").equals("1");
		rightobjectprotected = cmd.get("rightobjectprotected").equals("1");
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public long getId() {
		return id;
	}

	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	public boolean isLeftobjectprotected() {
		return leftobjectprotected;
	}

	public void setLeftobjectprotected(boolean leftobjectprotected) {
		this.leftobjectprotected = leftobjectprotected;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

//	public Boolean isRetainoncheckin() {
//		return retainoncheckin;
//	}

//	public void setRetainoncheckin(boolean retainoncheckin) {
//		this.retainoncheckin = retainoncheckin;
//	}

	public Boolean isRightobjectprotected() {
		return rightobjectprotected;
	}

	public void setRightobjectprotected(boolean rightobjectprotected) {
		this.rightobjectprotected = rightobjectprotected;
	}
	
	public void toXmlElement(Element root){
		Element rt = root.addElement("relationType");
		rt.addElement("id").addText(String.valueOf(getId()) );
		rt.addElement("name").addText(  LocalMessage.loc(getName()));
		rt.addElement("description").addText( LocalMessage.loc(getDescription()));
//		rt.addElement("retainOnCheckin").addText( retainoncheckin.toString());
		rt.addElement("rightobjectprotected").addText( rightobjectprotected.toString());
		rt.addElement("leftobjectprotected").addText( leftobjectprotected.toString());
	}

}
