// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007 Dr.-Ing. Boris Horner
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

package server.dao;

import server.Folder;
import server.ObjectType;
import server.User;
import server.data.ObjectSystemData;
import server.exceptions.CinnamonException;

import java.util.List;

public interface ObjectSystemDataDAO extends GenericDAO<ObjectSystemData, Long> {

	/**
	 * Return all OSDs with a predecessor ID.
	 * @param preID ID of the predecessor
	 * @return List<ObjectSystemData>
	 */
	List<ObjectSystemData> findAllByPredecessorID(ObjectSystemData predecessor);
	
	/**
	 * Return all OSDs with a parent ID.
	 * @param parentID ID of the parent
	 * @return List<ObjectSystemData>
	 */
	List<ObjectSystemData> findAllByParent(Long parentID);

	/**
	 * Find all OSDs whose indexed-time is "0000-00-00 00:00:00". Those are the ones
	 * whose index is not current.
	 * @param maxResults maximum number of results
	 * @return a List of all IndexTargets (limited by maxResults).
	 */
	List<ObjectSystemData> findIndexTargets(Integer maxResults);
	
	/**
	 * Set the indexed-column to 0 and trigger a re-indexing by the IndexServer.
	 */
	Integer prepareReIndex();
	
//	ObjectSystemData findByName(String name);

	List<ObjectSystemData> findAllByNameAndType(String name, ObjectType type);
	
	List<ObjectSystemData> findAllByRootAndNotSelf(ObjectSystemData osd);
	
	/**
	 * Convenience method: given a String which represents a OSD-Id, this
	 * method returns the OSD.
	 * @param id - String parsable with Long.parseLong
	 * @return ObjectSystemData
	 * @throws server.exceptions.CinnamonException if String is not a valid Long.
	 */
	ObjectSystemData get(String id);

	/**
	 * Find all versions of a given object.
	 * @param osd
	 * @return list of all objects of this object tree.
	 */
	List<ObjectSystemData> findAllVersions(ObjectSystemData osd);
	
	/**
	 * Find a specific version of an object, given the root object of this object tree.
	 * @param osd
	 * @return the requested version of null.
	 */
	ObjectSystemData findByRootAndVersion(ObjectSystemData osd, String version);

	List<ObjectSystemData> findAllVersionsOrderLastToFirst(ObjectSystemData osd);
	
	List<ObjectSystemData> findAllByTypeAndProcstateAndOwnerAndRelationToLeftOsd(ObjectType type, String procstate, User owner, ObjectSystemData related);
	List<ObjectSystemData> findAllByTypeAndProcstateAndRelationToLeftOsd(ObjectType type, String procstate, ObjectSystemData related);
	List<ObjectSystemData> findAllByTypeAndProcstateAndOwner(ObjectType type, String procstate, User owner);
	List<ObjectSystemData> findAllByTypeAndProcstate(ObjectType type, String procstate);
	
	List<ObjectSystemData> findAllByParentAndName(Folder parent, String name);
}
