package server.data;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.global.Conf;
import server.global.ConfThreadLocal;
import server.helpers.GeneralHelper;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.UUID;

public class ContentStore {
	
	static final String sep = File.separator;
	
	/**
	 * create a unique subfolder on in the repository and copy the file into it.  
	 * @param sourcePath path to the file to copy
	 * @param repository
	 * @return String
	 * @throws java.io.IOException
	 */
	// TODO: find a better method name
	public static String copyToContentStore(String sourcePath, String repository) throws IOException {
		Logger log = LoggerFactory.getLogger(server.data.ContentStore.class);
		Conf conf = ConfThreadLocal.getConf();
		
		File source = new File(sourcePath);
		String targetName = UUID.randomUUID().toString(); // GUID
		String subfolderName = getSubFolderName(targetName);

		// TODO: refactor identical code between upload&copy-tocontentstore.

		String subfolderPath = conf.getDataRoot() + repository + sep
						+ subfolderName;
		File subfolder = new File(subfolderPath);
		boolean result = subfolder.mkdirs();
		log.debug( "Result of mkdir: " + result);
		if (!result) {
			throw new IOException("Could not create directory "	+ subfolderPath);
		}

		String targetPath = subfolderPath + sep + targetName;
		GeneralHelper.copyFile(source, new File(targetPath));
		return subfolderName + sep + targetName;
	}

	private static String getSubFolderName(String f) {
		String subfolderName = f.substring(0, 2) + sep
				+ f.substring(2, 4) + sep + f.substring(4, 6);
		return subfolderName;
	}

	/**
	 * put the file into the content store subfolder.
	 * @param file
	 * @param repository
	 * @return String
	 * @throws java.io.IOException
	 */
	public static String upload(UploadedFile file,
			String repository) throws IOException {		
		Logger log = LoggerFactory.getLogger(server.data.ContentStore.class);
		Conf conf = ConfThreadLocal.getConf();
		
		File f = new File(file.getFileBufferPath());
	
		String subfolderName = getSubFolderName(f.getName());

		String subfolderPath = conf.getDataRoot() + repository
				+ sep + subfolderName;
		File subfolder = new File(subfolderPath);

		boolean result = subfolder.mkdirs();
		log.debug( "Result of mkdir: " + result);

		String contentPath = subfolderPath + sep + f.getName();

		result = f.renameTo(new File(contentPath));
		log.debug( "Result of renameTo(" + contentPath + ": "
				+ result);
		if (!result) {
			throw new IOException("Could not rename uploaded file "	+ contentPath);
		}
        else{
		    return subfolderName+ sep + f.getName();
        }
	}

	/**
	 * Delete a file corresponding to an OSD object from the harddisk (and its
	 * containing folder if it will empty after deletion)
	 * 
	 * @param osd
	 * @throws Exception
	 */
	public static void deleteObjectFile(ObjectSystemData osd) {
		if(osd.getContentPath() == null){
			return;
		}
		File contentFile = new File(osd.getContentPath());
		if (contentFile.exists()) { // cannot delete non-existant file
			File contentFileFolder = contentFile.getParentFile();
			if (!contentFile.delete()) {
				throw new RuntimeException("error.delete.content");
			}

			// TODO: delete all empty content folders above
			if (contentFileFolder.listFiles().length == 0) {
				boolean deleteSuccessful = contentFileFolder.delete();
				if (!deleteSuccessful) {
					throw new RuntimeException("error.delete.content");
				}
			}
		}
	}

	/**
	 * Create a byte array of the same size as the given OSD's content and set the
	 * HttpServletResponse to binary/octed-stream for a binary attachment with name
	 * = osd.getName().
	 * @param res HttpServletResonse 
	 * @param osd the object which is the source for the file content.
	 * @return an empty byte-array of size=osd.content.size
	 * @throws java.io.FileNotFoundException
	 */
	public static byte[] getObjectFileContent(HttpServletResponse res,
			ObjectSystemData osd){
		Long contentSize = osd.getContentSize();
		int fileSize = contentSize.intValue();
		// TODO: buffered streaming
		byte b[] = new byte[fileSize];
		res.setContentLength(fileSize);
		res.setContentType("binary/octet-stream"); // TODO: softcode with
													// detailed type info
		res.setHeader("Content-Disposition", "attachment; filename="
				+ osd.getName());
		return b;
	}

}
