package server.i18n;

import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.*;
import java.io.Serializable;

/*
 * note: this class is called CinnamonLocale so it will not interfere with the 
 * default Java-class Locale which will probably be used in the same context.
 */

@Entity
@Table(name = "messages",
		uniqueConstraints = {@UniqueConstraint(columnNames={"message", "language_id"})}		
)
public class Message implements Serializable{

	private static final long	serialVersionUID	= 1L;
	
	@SuppressWarnings("unused")
	private transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Id	@GeneratedValue
	@Column(name = "id")
	private long id;
	
	@Column(name = "message",
			nullable = false)
	private String message;
	
	@ManyToOne
	@JoinColumn(name = "language_id",
			nullable = false)
	private Language language;
	
	@Column(name = "translation",
			nullable=false)
	private String translation;
	
	
	public Message(){}
	
	public Message(String message, Language language, String translation){
		this.message = message;
		this.language = language;
		this.translation = translation;
	}

	/**
	 * @return the id
	 */
	public long getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	/**
	 * @return the language
	 */
	public Language getLanguage() {
		return language;
	}

	/**
	 * @param language the language to set
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * @return the message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @param message the message to set
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * @return the translation
	 */
	public String getTranslation() {
		return translation;
	}

	/**
	 * @param translation the translation to set
	 */
	public void setTranslation(String translation) {
		this.translation = translation;
	}
	
	public void toXmlElement(Element root){
		Element msg = root.addElement("message");
		msg.addElement("id").addText(String.valueOf(id));
		msg.addElement("message").addText( message);
		msg.addElement("languageId").addText( String.valueOf(language.getId()));
		msg.addElement("translation").addText( translation );
	}
	
}
