package server.index;

import org.apache.lucene.document.Document;
import server.interfaces.XmlConvertable;

import java.util.Set;

/**
 * A ResultValidator checks if the results are correct and may be given to
 * the user. For Cinnamon, it needs to check the permissions of a given
 * object.<br/>
 * @author ingo
 *
 */
public interface ResultValidator {

	/**
	 * Check whether the Lucene document may be included in the result list.
	 * If the object has to bee excluded from the result set for any reason,
	 * validateSearchResult returns false (and not an exception, though those
	 * should be logged).
	 * @param doc the Document to check
	 * @return true if the document may be included in the result list,
	 * false otherwise.
	 */
	Boolean validateAccessPermissions(Document doc, Class<? extends Indexable> filterClass, Set<XmlConvertable> resultStore);
	
}
