package server.index.indexer;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.dom4j.Node;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.Folder;
import server.dao.DAOFactory;
import server.dao.FolderDAO;
import utils.HibernateSession;
import utils.ParamParser;

import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Index the parent folder path of an object or folder.
 *
 */
public class ParentFolderPathIndexer extends DefaultIndexer{

	transient Logger log = LoggerFactory.getLogger(this.getClass());
	static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	public ParentFolderPathIndexer() {
		index = Index.NOT_ANALYZED;
		store = Store.NO;
	}
	
	@SuppressWarnings("unchecked")
	public void indexObject(String data, Document doc, String fieldname,
			String searchString, Boolean multipleResults) {

		org.dom4j.Document indexObject = ParamParser.parseXmlToDocument(data, "error.parse.indexable_object");
		
		List<Node> hits = new ArrayList<Node>();
		
		if(multipleResults){
			hits = indexObject.selectNodes(searchString);
		}
		else{
			Node node = indexObject.selectSingleNode(searchString);
			if(node != null){
				hits.add(node);
			}
		}
		
		for(Node node : hits){
			String nodeValue = convertNodeToString(node);
			if(nodeValue != null){
								
				// fieldValue should be: osd.parent or folder.parent
				EntityManager em = HibernateSession.getLocalEntityManager();								
				FolderDAO folderDao = daoFactory.getFolderDAO(em);
				Folder folder = folderDao.get(nodeValue);

				/*
				 * getParentFolders returns: "c/b/a" for folders /a/b/c 
				 */
				List<Folder> folders = folderDao.getParentFolders(folder); 				
				StringBuilder path = new StringBuilder();
				Collections.reverse(folders);
				folders.add(folder);
				for(Folder f : folders){
					path.append('/');	
					path.append(f.getName());					
				}
				log.debug("fieldname: "+fieldname+" value: "+ path.toString());
				doc.add(new Field(fieldname, path.toString().toLowerCase(), store, index));				
			}
			else{
				log.debug("nodeValue == null");
			}
		} 	
	}
		
}
