package server.lifecycle;

import org.dom4j.Element;
import server.global.Constants;
import server.i18n.LocalMessage;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

/**

 */
@Entity
@Table(name = "lifecycles",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}
)
public class LifeCycle implements Serializable{

    @Id
    @GeneratedValue
	@Column(name = "id")
	private long 	id;

	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;

    @OneToMany(mappedBy = "lifeCycle")
    private Set<LifeCycleState> states = new HashSet<LifeCycleState>();

    @ManyToOne
    @JoinColumn(name = "default_state_id",
            nullable = true)
    private LifeCycleState defaultState;

    public LifeCycle() {
    }

    public LifeCycle(String name, LifeCycleState defaultState) {
        this.name = name;
        this.defaultState = defaultState;
    }

    public void toXmlElement(Element root){
		Element lc = root.addElement("lifecycle");
		lc.addElement("id").addText(String.valueOf(id));
		lc.addElement("name").addText( LocalMessage.loc(name) );
        Element ds = lc.addElement("defaultState");
        if(defaultState != null){
            defaultState.toXmlElement(ds);
        }
        Element lstates = lc.addElement("states");
        for(LifeCycleState lcs : states){
            lcs.toXmlElement(lstates);
        }
	}

    public Set<LifeCycleState> getStates() {
        return states;
    }

    public void setStates(Set<LifeCycleState> states) {
        this.states = states;
    }

    public long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public LifeCycleState getDefaultState() {
        return defaultState;
    }

    public void setDefaultState(LifeCycleState defaultState) {
        this.defaultState = defaultState;
    }
}
