package server.lifecycle;

import org.dom4j.Element;
import org.hibernate.annotations.Type;
import server.Format;
import server.dao.DAOFactory;
import server.global.Constants;
import server.i18n.LocalMessage;
import server.lifecycle.LifeCycle;
import server.transformation.ITransformer;

import javax.persistence.*;
import java.io.Serializable;

/**
 *
 */
@Entity
@Table(name = "lifecycle_states",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}
)
public class LifeCycleState implements Serializable{

    	private static final long	serialVersionUID	= 1L;

	static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);

	@Id
    @GeneratedValue
	@Column(name = "id")
	private long 	id;

	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;

	@Column(name = "state_class",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private Class<? extends IState> stateClass;

   	@ManyToOne
	@JoinColumn(name = "lifecycle_id",
			nullable = false
	)
	private LifeCycle lifeCycle;

    @Column(name = "parameter",
			length = Constants.XPATH_LENGTH,
			nullable = false)
	@Type(type="text")
	String parameter = "";

    public LifeCycleState(){

    }

    public LifeCycleState(String name, Class<? extends IState> stateClass, String parameter, LifeCycle lifeCycle) {
        this.name = name;
        this.stateClass = stateClass;
        this.parameter = parameter;
        this.lifeCycle = lifeCycle;
    }

    public void toXmlElement(Element root){
		Element lcs = root.addElement("lifecycleState");
		lcs.addElement("id").addText(String.valueOf(id));
		lcs.addElement("name").addText( LocalMessage.loc(name));
        lcs.addElement("stateClass").addText(stateClass.getName());
        lcs.addElement("parameter").addText(parameter);
	}

    public long getId() {
        return id;
    }

    private void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Class<? extends IState> getStateClass() {
        return stateClass;
    }

    public void setStateClass(Class<? extends IState> stateClass) {
        this.stateClass = stateClass;
    }

    public LifeCycle getLifeCycle() {
        return lifeCycle;
    }

    public void setLifeCycle(LifeCycle lifeCycle) {
        this.lifeCycle = lifeCycle;
    }

    public String getParameter() {
        return parameter;
    }

    public void setParameter(String parameter) {
        this.parameter = parameter;
    }
}
