package server.trigger.test;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.DAOFactory;
import server.dao.HibernateDAOFactory;
import server.dao.ObjectSystemDataDAO;
import server.data.ObjectSystemData;
import server.interfaces.Repository;
import server.interfaces.Response;
import server.trigger.ITrigger;
import utils.HibernateSession;

import java.util.Map;

/**
 * Test class for testing the ChangeTrigger implementation of CinnamonServer.
 * It will change a given object with name "ChangeTriggerTestObject" 
 * by adding a "<testTrigger/>"-node to the metadata, if applicable.
 *
 */
public class TestTrigger implements ITrigger {
	
	private final transient Logger log = LoggerFactory.getLogger(this.getClass());
	static DAOFactory daoFactory = DAOFactory.instance(HibernateDAOFactory.class);
	
	public TestTrigger(){}

	@Override
	public void executePostCommand(String command,
			Map<String, Object> params, Repository repository, Response response) {
	
		ObjectSystemDataDAO osdDao = daoFactory.getObjectSystemDataDAO(
				HibernateSession.getLocalEntityManager());
		if(! params.containsKey("id")){
			log.debug("no id found");
			return;
		}
			
		/*
		 *  Note: in a production system, id could map to a folder-id or something else,
		 *  which only accidentally also maps to an object with the same id.
		 *  You should check if params contains an appropriate command (like setContent).  
		 */
		ObjectSystemData osd = osdDao.get((String)params.get("id"));
		if(osd != null){
			log.debug("found OSD for postCommandTrigger");
			if(osd.getName().equals("ChangeTriggerTestObject")){
				log.debug("changing metadata");
				String meta = osd.getMetadata();
				log.debug("meta before: '"+meta+"'");
				meta = meta.replace("<meta/>", "<meta><testTrigger/><name>SearchTestTriggerName</name></meta>");
				log.debug("meta after: '"+meta+"'");
				osd.setMetadata(meta);
				log.debug("updating changed object in index.");
				synchronized(repository.getLuceneBridge()){
					repository.getLuceneBridge().updateObjectInIndex(osd);
				}
			}
		}
	}

	@Override
	public void executePreCommand(String command,
			Map<String, Object> params, Repository repository) {
		log.debug("nothing to do in TestTrigger.executePreCommand.");
	}



}
